@@ -86,6 +86,65 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (CallbackHandler)
 };
 
+//==============================================================================
+// This is an AudioTransportSource which will own it's assigned source
+struct AudioSourceOwningTransportSource  : public AudioTransportSource
+{
+    AudioSourceOwningTransportSource (PositionableAudioSource* s)  : source (s)
+    {
+        AudioTransportSource::setSource (s);
+    }
+
+    ~AudioSourceOwningTransportSource()
+    {
+        setSource (nullptr);
+    }
+
+private:
+    ScopedPointer<PositionableAudioSource> source;
+
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioSourceOwningTransportSource)
+};
+
+//==============================================================================
+// An AudioSourcePlayer which will remove itself from the AudioDeviceManager's
+// callback list once it finishes playing its source
+struct AutoRemovingSourcePlayer  : public AudioSourcePlayer,
+                                   private Timer
+{
+    AutoRemovingSourcePlayer (AudioDeviceManager& dm, AudioTransportSource* ts, bool ownSource)
+        : manager (dm), transportSource (ts, ownSource)
+    {
+        jassert (ts != nullptr);
+        manager.addAudioCallback (this);
+        AudioSourcePlayer::setSource (transportSource);
+        startTimerHz (10);
+    }
+
+    ~AutoRemovingSourcePlayer()
+    {
+        setSource (nullptr);
+        manager.removeAudioCallback (this);
+    }
+
+    void timerCallback() override
+    {
+        if (getCurrentSource() == nullptr || ! transportSource->isPlaying())
+            delete this;
+    }
+
+    void audioDeviceStopped() override
+    {
+        AudioSourcePlayer::audioDeviceStopped();
+        setSource (nullptr);
+    }
+
+private:
+    AudioDeviceManager& manager;
+    OptionalScopedPointer<AudioTransportSource> transportSource;
+
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AutoRemovingSourcePlayer)
+};
 
 //==============================================================================
 AudioDeviceManager::AudioDeviceManager()
@@ -103,8 +162,11 @@ AudioDeviceManager::~AudioDeviceManager()
 {
     currentAudioDevice = nullptr;
     defaultMidiOutput = nullptr;
-}
 
+    for (int i = 0; i < callbacks.size(); ++i)
+        if (AutoRemovingSourcePlayer* p = dynamic_cast<AutoRemovingSourcePlayer*> (callbacks.getUnchecked(i)))
+            delete p;
+}
 
 //==============================================================================
 void AudioDeviceManager::createDeviceTypesIfNeeded()
@@ -284,8 +346,8 @@ String AudioDeviceManager::initialiseFromXML (const XmlElement& xml,
             currentDeviceType = availableDeviceTypes.getUnchecked(0)->getTypeName();
     }
 
-    setup.bufferSize = xml.getIntAttribute ("audioDeviceBufferSize");
-    setup.sampleRate = xml.getDoubleAttribute ("audioDeviceRate");
+    setup.bufferSize = xml.getIntAttribute ("audioDeviceBufferSize", setup.bufferSize);
+    setup.sampleRate = xml.getDoubleAttribute ("audioDeviceRate", setup.sampleRate);
 
     setup.inputChannels .parseString (xml.getStringAttribute ("audioDeviceInChans",  "11"), 2);
     setup.outputChannels.parseString (xml.getStringAttribute ("audioDeviceOutChans", "11"), 2);
@@ -926,143 +988,16 @@ void AudioDeviceManager::setDefaultMidiOutput (const String& deviceName)
     }
 }
 
-//==============================================================================
-// This is an AudioTransportSource which will own it's assigned source
-class AudioSourceOwningTransportSource  : public AudioTransportSource
-{
-public:
-    AudioSourceOwningTransportSource() {}
-    ~AudioSourceOwningTransportSource()  { setSource (nullptr); }
-
-    void setSource (PositionableAudioSource* newSource)
-    {
-        if (src != newSource)
-        {
-            ScopedPointer<PositionableAudioSource> oldSourceDeleter (src);
-            src = newSource;
-
-            // tell the base class about the new source before deleting the old one
-            AudioTransportSource::setSource (newSource);
-        }
-    }
-
-private:
-    ScopedPointer<PositionableAudioSource> src;
-
-    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioSourceOwningTransportSource)
-};
-
-//==============================================================================
-// An Audio player which will remove itself from the AudioDeviceManager's
-// callback list once it finishes playing its source
-class AutoRemovingSourcePlayer  : public AudioSourcePlayer,
-                                  private ChangeListener
-{
-public:
-    struct DeleteOnMessageThread  : public CallbackMessage
-    {
-        DeleteOnMessageThread (AutoRemovingSourcePlayer* p)  : parent (p) {}
-        void messageCallback() override     { delete parent; }
-
-        AutoRemovingSourcePlayer* parent;
-    };
-
-    //==============================================================================
-    AutoRemovingSourcePlayer (AudioDeviceManager& deviceManager, bool ownSource)
-        : manager (deviceManager),
-          deleteWhenDone (ownSource),
-          hasAddedCallback (false),
-          recursiveEntry (false)
-    {
-    }
-
-    void changeListenerCallback (ChangeBroadcaster* newSource) override
-    {
-        if (AudioTransportSource* currentTransport
-               = dynamic_cast<AudioTransportSource*> (getCurrentSource()))
-        {
-            ignoreUnused (newSource);
-            jassert (newSource == currentTransport);
-
-            if (! currentTransport->isPlaying())
-            {
-                // this will call audioDeviceStopped!
-                manager.removeAudioCallback (this);
-            }
-            else if (! hasAddedCallback)
-            {
-                hasAddedCallback = true;
-                manager.addAudioCallback (this);
-            }
-        }
-    }
-
-    void audioDeviceStopped() override
-    {
-        if (! recursiveEntry)
-        {
-            ScopedValueSetter<bool> s (recursiveEntry, true, false);
-
-            manager.removeAudioCallback (this);
-            AudioSourcePlayer::audioDeviceStopped();
-
-            if (MessageManager* mm = MessageManager::getInstanceWithoutCreating())
-            {
-                if (mm->isThisTheMessageThread())
-                    delete this;
-                else
-                    (new DeleteOnMessageThread (this))->post();
-            }
-        }
-    }
-
-    void setSource (AudioTransportSource* newSource)
-    {
-        AudioSource* oldSource = getCurrentSource();
-
-        if (AudioTransportSource* oldTransport = dynamic_cast<AudioTransportSource*> (oldSource))
-            oldTransport->removeChangeListener (this);
-
-        if (newSource != nullptr)
-            newSource->addChangeListener (this);
-
-        AudioSourcePlayer::setSource (newSource);
-
-        if (deleteWhenDone)
-            delete oldSource;
-    }
-
-private:
-    // only allow myself to be deleted when my audio callback has been removed
-    ~AutoRemovingSourcePlayer()
-    {
-        setSource (nullptr);
-    }
-
-    AudioDeviceManager& manager;
-    bool deleteWhenDone, hasAddedCallback, recursiveEntry;
-
-    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AutoRemovingSourcePlayer)
-};
-
 //==============================================================================
 // An AudioSource which simply outputs a buffer
-class AudioSampleBufferSource : public PositionableAudioSource
+class AudioSampleBufferSource  : public PositionableAudioSource
 {
 public:
-    AudioSampleBufferSource (AudioSampleBuffer* audioBuffer, bool shouldLoop, bool ownBuffer)
-        : position (0),
-          buffer (audioBuffer),
-          looping (shouldLoop),
-          deleteWhenDone (ownBuffer)
+    AudioSampleBufferSource (AudioSampleBuffer* audioBuffer, bool ownBuffer, bool playOnAllChannels)
+        : buffer (audioBuffer, ownBuffer),
+          position (0), looping (false), playAcrossAllChannels (playOnAllChannels)
     {}
 
-    ~AudioSampleBufferSource()
-    {
-        if (deleteWhenDone)
-            delete buffer;
-    }
-
     //==============================================================================
     void setNextReadPosition (int64 newPosition) override
     {
@@ -1074,69 +1009,48 @@ public:
         position = jmin (buffer->getNumSamples(), static_cast<int> (newPosition));
     }
 
-    int64 getNextReadPosition() const override
-    {
-        return static_cast<int64> (position);
-    }
+    int64 getNextReadPosition() const override      { return static_cast<int64> (position); }
+    int64 getTotalLength() const override           { return static_cast<int64> (buffer->getNumSamples()); }
 
-    int64 getTotalLength() const override
-    {
-        return static_cast<int64> (buffer->getNumSamples());
-    }
-
-    bool isLooping() const override
-    {
-        return looping;
-    }
-
-    void setLooping (bool shouldLoop) override
-    {
-        looping = shouldLoop;
-    }
+    bool isLooping() const override                 { return looping; }
+    void setLooping (bool shouldLoop) override      { looping = shouldLoop; }
 
     //==============================================================================
-    void prepareToPlay (int samplesPerBlockExpected, double sampleRate) override
-    {
-        ignoreUnused (samplesPerBlockExpected, sampleRate);
-    }
-
-    void releaseResources() override
-    {}
+    void prepareToPlay (int, double) override {}
+    void releaseResources() override {}
 
     void getNextAudioBlock (const AudioSourceChannelInfo& bufferToFill) override
     {
-        int max = jmin (buffer->getNumSamples() - position, bufferToFill.numSamples);
+        bufferToFill.clearActiveBufferRegion();
 
-        jassert (max >= 0);
+        const int bufferSize = buffer->getNumSamples();
+        const int samplesNeeded = bufferToFill.numSamples;
+        const int samplesToCopy = jmin (bufferSize - position, samplesNeeded);
+
+        if (samplesToCopy > 0)
         {
-            int ch;
             int maxInChannels = buffer->getNumChannels();
-            int maxOutChannels = jmin (bufferToFill.buffer->getNumChannels(),
-                                       jmax (maxInChannels, 2));
+            int maxOutChannels = bufferToFill.buffer->getNumChannels();
 
-            for (ch = 0; ch < maxOutChannels; ch++)
-            {
-                int inChannel = ch % maxInChannels;
+            if (! playAcrossAllChannels)
+                maxOutChannels = jmin (maxOutChannels, maxInChannels);
 
-                if (max > 0)
-                    bufferToFill.buffer->copyFrom (ch, bufferToFill.startSample, *buffer, inChannel, position, max);
-            }
-
-            for (; ch < bufferToFill.buffer->getNumChannels(); ++ch)
-                bufferToFill.buffer->clear (ch, bufferToFill.startSample, bufferToFill.numSamples);
+            for (int i = 0; i < maxOutChannels; ++i)
+                bufferToFill.buffer->copyFrom (i, bufferToFill.startSample, *buffer,
+                                               i % maxInChannels, position, samplesToCopy);
         }
 
-        position += max;
+        position += samplesNeeded;
 
         if (looping)
-            position = position % buffer->getNumSamples();
+            position %= bufferSize;
     }
 
 private:
     //==============================================================================
+    OptionalScopedPointer<AudioSampleBuffer> buffer;
     int position;
-    AudioSampleBuffer* buffer;
-    bool looping, deleteWhenDone;
+    bool looping, playAcrossAllChannels;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioSampleBufferSource)
 };
@@ -1165,38 +1079,38 @@ void AudioDeviceManager::playSound (const void* resourceData, size_t resourceSiz
 
 void AudioDeviceManager::playSound (AudioFormatReader* reader, bool deleteWhenFinished)
 {
-    playSound (new AudioFormatReaderSource (reader, deleteWhenFinished), true);
+    if (reader != nullptr)
+        playSound (new AudioFormatReaderSource (reader, deleteWhenFinished), true);
+}
+
+void AudioDeviceManager::playSound (AudioSampleBuffer* buffer, bool deleteWhenFinished, bool playOnAllOutputChannels)
+{
+    if (buffer != nullptr)
+        playSound (new AudioSampleBufferSource (buffer, deleteWhenFinished, playOnAllOutputChannels), true);
 }
 
 void AudioDeviceManager::playSound (PositionableAudioSource* audioSource, bool deleteWhenFinished)
 {
     if (audioSource != nullptr && currentAudioDevice != nullptr)
     {
-        if (AudioTransportSource* transport = dynamic_cast<AudioTransportSource*> (audioSource))
-        {
-            AutoRemovingSourcePlayer* player = new AutoRemovingSourcePlayer (*this, deleteWhenFinished);
-            player->setSource (transport);
-        }
-        else
-        {
-            AudioTransportSource* transportSource;
+        AudioTransportSource* transport = dynamic_cast<AudioTransportSource*> (audioSource);
 
+        if (transport == nullptr)
+        {
             if (deleteWhenFinished)
             {
-                AudioSourceOwningTransportSource* owningTransportSource = new AudioSourceOwningTransportSource();
-                owningTransportSource->setSource (audioSource);
-                transportSource = owningTransportSource;
+                transport = new AudioSourceOwningTransportSource (audioSource);
             }
             else
             {
-                transportSource = new AudioTransportSource;
-                transportSource->setSource (audioSource);
+                transport = new AudioTransportSource();
+                transport->setSource (audioSource);
+                deleteWhenFinished = true;
             }
-
-            // recursively call myself
-            playSound (transportSource, true);
-            transportSource->start();
         }
+
+        transport->start();
+        new AutoRemovingSourcePlayer (*this, transport, deleteWhenFinished);
     }
     else
     {
@@ -1205,11 +1119,6 @@ void AudioDeviceManager::playSound (PositionableAudioSource* audioSource, bool d
     }
 }
 
-void AudioDeviceManager::playSound (AudioSampleBuffer* buffer, bool deleteWhenFinished)
-{
-    playSound (new AudioSampleBufferSource (buffer, false, deleteWhenFinished), true);
-}
-
 void AudioDeviceManager::playTestSound()
 {
     const double sampleRate = currentAudioDevice->getCurrentSampleRate();
@@ -1228,7 +1137,7 @@ void AudioDeviceManager::playTestSound()
     newSound->applyGainRamp (0, 0, soundLength / 10, 0.0f, 1.0f);
     newSound->applyGainRamp (0, soundLength - soundLength / 4, soundLength / 4, 1.0f, 0.0f);
 
-    playSound (newSound, true);
+    playSound (newSound, true, true);
 }
 
 //==============================================================================
