@@ -1126,6 +1126,15 @@ bool AudioProcessorGraph::removeNode (const uint32 nodeId)
     return false;
 }
 
+bool AudioProcessorGraph::removeNode (Node* node)
+{
+    if (node != nullptr)
+        return removeNode (node->nodeId);
+
+    jassertfalse;
+    return false;
+}
+
 //==============================================================================
 const AudioProcessorGraph::Connection* AudioProcessorGraph::getConnectionBetween (const uint32 sourceNodeId,
                                                                                   const int sourceChannelIndex,
@@ -1460,7 +1469,6 @@ void AudioProcessorGraph::processAudio (AudioBuffer<FloatType>& buffer, MidiBuff
     midiMessages.addEvents (currentMidiOutputBuffer, 0, buffer.getNumSamples(), 0);
 }
 
-bool AudioProcessorGraph::silenceInProducesSilenceOut() const       { return false; }
 double AudioProcessorGraph::getTailLengthSeconds() const            { return 0; }
 bool AudioProcessorGraph::acceptsMidi() const                       { return true; }
 bool AudioProcessorGraph::producesMidi() const                      { return true; }
@@ -1601,11 +1609,6 @@ void AudioProcessorGraph::AudioGraphIOProcessor::processBlock (AudioBuffer<doubl
     processAudio (buffer, midiMessages);
 }
 
-bool AudioProcessorGraph::AudioGraphIOProcessor::silenceInProducesSilenceOut() const
-{
-    return isOutput();
-}
-
 double AudioProcessorGraph::AudioGraphIOProcessor::getTailLengthSeconds() const
 {
     return 0;
