@@ -172,7 +172,7 @@ String XmlDocument::getFileContents (const String& filename) const
             return in->readEntireStreamAsString();
     }
 
-    return String::empty;
+    return String();
 }
 
 juce_wchar XmlDocument::readNextChar() noexcept
@@ -617,9 +617,17 @@ void XmlDocument::readChildElements (XmlElement& parent)
                 }
                 else
                 {
-                    for (;;)
+                    for (;; ++input)
                     {
-                        const juce_wchar nextChar = *input;
+                        juce_wchar nextChar = *input;
+
+                        if (nextChar == '\r')
+                        {
+                            nextChar = '\n';
+
+                            if (input[1] == '\n')
+                                continue;
+                        }
 
                         if (nextChar == '<' || nextChar == '&')
                             break;
@@ -633,7 +641,6 @@ void XmlDocument::readChildElements (XmlElement& parent)
 
                         textElementContent.appendUTF8Char (nextChar);
                         contentShouldBeUsed = contentShouldBeUsed || ! CharacterFunctions::isWhitespace (nextChar);
-                        ++input;
                     }
                 }
             }
