@@ -773,8 +773,8 @@ public:
     }
 
     /** Returns the smallest integer-aligned rectangle that completely contains this one.
-        This is only relevent for floating-point rectangles, of course.
-        @see toFloat()
+        This is only relevant for floating-point rectangles, of course.
+        @see toFloat(), toNearestInt()
     */
     Rectangle<int> getSmallestIntegerContainer() const noexcept
     {
@@ -786,6 +786,17 @@ public:
         return Rectangle<int> (x1, y1, x2 - x1, y2 - y1);
     }
 
+    /** Casts this rectangle to a Rectangle<int>.
+        This uses roundToInt to snap x, y, width and height to the nearest integer (losing precision).
+        If the rectangle already uses integers, this will simply return a copy.
+        @see getSmallestIntegerContainer()
+    */
+    Rectangle<int> toNearestInt() const noexcept
+    {
+        return Rectangle<int> (roundToInt (pos.x), roundToInt (pos.y),
+                               roundToInt (w),     roundToInt (h));
+    }
+
     /** Casts this rectangle to a Rectangle<float>.
         @see getSmallestIntegerContainer
     */
@@ -914,8 +925,8 @@ private:
     Point<ValueType> pos;
     ValueType w, h;
 
-    static int parseIntAfterSpace (StringRef s) noexcept
-        { return s.text.findEndOfWhitespace().getIntValue32(); }
+    static ValueType parseIntAfterSpace (StringRef s) noexcept
+        { return static_cast<ValueType> (s.text.findEndOfWhitespace().getIntValue32()); }
 
     void copyWithRounding (Rectangle<int>& result) const noexcept    { result = getSmallestIntegerContainer(); }
     void copyWithRounding (Rectangle<float>& result) const noexcept  { result = toFloat(); }
